﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.IO;

namespace ych_sdk.Sample
{
    public abstract class BaseTest
    {
        [DataContract]
        private class RspData
        {
            private string m_result;

            [DataMember]
            public string result
            {
                get { return m_result; }
                set { m_result = value; }
            }
        }

        protected String appkey = "";

        protected String secret = "";

        protected String topAppkey = ""; // 如果没有TOP appkey，可以直接使用御城河appkey填充，即topAppkey=appkey

        protected String apiHost = "http://gw.ose.aliyun.com/event/"; // 日志接入API

        protected String accountHost = "http://account.ose.aliyun.com/"; // 账号风控API

        protected SortedDictionary<String, String> defaultParam = new SortedDictionary<String, String>();

        public BaseTest()
        {
            if (string.IsNullOrEmpty(appkey)
                || string.IsNullOrEmpty(secret))
            {
                throw new Exception("appkey or secret is empty.");
            }

            if (string.IsNullOrEmpty(topAppkey))
            {
                topAppkey = appkey;
            }

            DateTime startTime = TimeZone.CurrentTimeZone.ToLocalTime(new DateTime(1970, 1, 1));
            DateTime now = DateTime.Now;
            long time = (long)(now - startTime).TotalSeconds * 1000;

            defaultParam.Add("appName", "我的测试应用");
            defaultParam.Add("ntime", time.ToString());
            defaultParam.Add("time", now.ToString("yyyy-MM-dd HH:mm:ss"));
            defaultParam.Add("ati", "123456789123456789");
            defaultParam.Add("userId", "tester");
            defaultParam.Add("userIp", "42.120.74.9");
        }

        protected void parseResult(String response)
        {
            RspData data = null;

            try
            {
                data = FromJson<RspData>(response);
            }
            catch (Exception)
            {
            }

            if (data != null && data.result.Equals("success"))
            {
                Console.WriteLine("[" + GetType().Name + "] success.");
            }
            else
            {
                Console.WriteLine("[" + GetType().Name + "] " + response);
            }
        }

        protected String ObjectToJson(object obj)
        {
            DataContractJsonSerializer serializer = new DataContractJsonSerializer(obj.GetType());
            MemoryStream stream = new MemoryStream();
            serializer.WriteObject(stream, obj);
            string json = Encoding.UTF8.GetString(stream.ToArray());
            stream.Close();
            return json;
        }

        protected T FromJson<T>(String strJson) where T : class
        {
            DataContractJsonSerializer ds = new DataContractJsonSerializer(typeof(T));
            MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(strJson));
            return ds.ReadObject(ms) as T;
        }

        public abstract void exec();
    }
}
